<?php namespace Manage\Lib;
use Dever;
# 通用页面
class Page extends Auth
{
    protected $db;
    protected $key;
    protected $id = 0;
    protected $input = false;
    protected $recycler = false;
    protected $menu = array();
    protected $config = array();
    protected $field = array();
    public $info = array();
    public function __construct($key = '', $load = '', $input = true, $config = array())
    {
        parent::__construct();
        $this->key = $key;
        $this->input = $input;
        $this->id = $this->input('id', 0);
        if (!$load) {
            $load = Dever::input('load');
        }
        list($this->db, $this->menu) = Dever::load('common', 'manage')->db($load);
        if ($this->menu && $this->menu['show'] == 1) {
            $this->checkMenu($this->menu['id'], false);
        }
        $this->config = $this->db->config['manage'][$key] ?? array();
        if ($config) {
            $this->config = array_merge($this->config, $config);
        }
        if ($this->id && !strstr($this->id, ',')) {
            $this->info = $this->db->find($this->id);
        }
    }

    public function setting($key, &$data, $struct = true, $type = 'show', $disable = false)
    {
        if (empty($this->config[$key]) && $struct && isset($this->db->config['struct']) && $this->db->config['struct']) {
            $this->config[$key] = $this->db->config['struct'];
        }
        if (empty($this->config[$key])) {
            return;
        }
        $setting = $this->config[$key];
        if (is_string($setting)) {
            $setting = explode(',', $setting);
        }
        $field = $this->input('field', '');
        if ($field && is_string($field) && strstr($field, 'dever_')) {
            $field = '';
        }
        return $this->setData($setting, $data, $field, $type, $disable);
    }

    # 获取某个数据的具体展示值
    public function getValue($key, $value, $data, $field = array())
    {
        if ($key == 'cdate') {
            $this->db->config['manage']['update']['field'][$key]['type'] = 'date';
        }
        $update = $this->db->config['manage']['update']['field'] ?? array();
        if ($show = Dever::issets($field, 'show')) {
            $value = $this->getShow($show, $data);
        } elseif ($value && isset($this->db->config['struct'][$key]['value']) && $this->db->config['struct'][$key]['value']) {
            $value = $this->db->value($key, $value);
        } elseif ($value && (isset($update[$key]) && isset($update[$key]['type']) && $update[$key]['type'] == 'date')) {
            if (isset($update[$key]['date_type']) && $update[$key]['date_type'] == 'year') {
                if ($update[$key]['date_type'] == 'year') {
                    $value = date('Y', $value);
                } elseif ($update[$key]['date_type'] == 'month') {
                    $value = date('Ym', $value);
                } else {
                    $value = date('Ymd', $value);
                }
            } else {
                if (strstr($value, 'T')) {
                    $value = date('Y-m-d H:i:s', strtotime($value));
                } elseif (is_numeric($value)) {
                    $value = date('Y-m-d H:i:s', $value);
                } else {
                    $value = '-';
                }
            }
        }
        
        return $value;
    }

    # 获取关联数据
    public function getOther($key, $set, $data)
    {
        $where = $config = array();
        if (isset($set['where'])) {
            foreach ($set['where'] as $k => $v) {
                if (!is_array($v) && isset($data[$v])) {
                    $where[$k] = $data[$v];
                } else {
                    $where[$k] = $v;
                }
            }
        }
        if (isset($set['col'])) {
            $config['col'] = $set['col'];
        }
        if ($where) {
            return Dever::db($key)->select($where, $config);
        }
        return array();
    }

    public function getShow($show, $data)
    {
        return \Dever\Helper\Str::val($show, $data);
    }

    # 获取菜单标题
    public function getTitle()
    {
        return $this->menu['name'];
    }

    # 获取左侧分栏
    protected function column(&$data, $name = '左侧分栏')
    {
        $data['column'] = false;
        if (isset($this->config['column'])) {
            if (empty($this->config['column']['hidden'])) {
                $data['column'] = $this->config['column'];
                if (isset($this->config['column']['add'])) {
                    $data['column']['add'] = array('name' => $this->config['column']['add'], 'func' => $this->getFunc('column_add', $name . '-' . $this->config['column']['add'], 101));
                }
                if (isset($this->config['column']['edit'])) {
                    $data['column']['edit'] = array('name' => '编辑', 'func' => $this->getFunc('column_edit', $name . '-编辑', 102));
                }
                if (isset($this->config['column']['delete'])) {
                    $data['column']['delete'] = array('name' => '删除', 'func' => $this->getFunc('column_delete', $name . '-删除', 103));
                }
                $data['column']['data'] = $this->config['column']['data'];
                if (is_string($data['column']['data'])) {
                    $data['column']['data'] = Dever::call($data['column']['data']);
                }
                $data['height'] = '100%';
            }
            
            if (isset($this->config['column']['active']) && $this->config['column']['where'] == 'id') {
                return $this->config['column']['active'];
            }
        }
    }

    # 通用的规则验证 一般为更新数据时使用
    protected function checkRules($set, $data)
    {
        if ($set['rules']) {
            if (!is_array($set['rules'])) {
                $set['rules'] = array
                (
                    array
                    (
                        'required' => true,
                        'trigger' => 'blur',
                        'message' => $set['name'] . '不能为空',
                    ),
                );
            }
            foreach ($set['rules'] as $k => $v) {
                if (isset($v['required']) && $v['required'] && !$data && $data !== 0) {
                    Dever::error($v['message']);
                }
                if ($data || $data === 0) {
                    if (isset($v['pattern']) && $v['pattern'] && !preg_match('/' . $v['pattern'] . '/', $data)) {
                        Dever::error($v['message']);
                    }
                    if (isset($v['type']) && $v['type']) {
                        if ($v['type'] == 'number' && !is_numeric($data)) {
                            Dever::error($v['message']);
                        } elseif ($v['type'] == 'array' && !is_array($data)) {
                            Dever::error($v['message']);
                        } elseif ($v['type'] == 'integer' && !is_int($data)) {
                            Dever::error($v['message']);
                        } elseif ($v['type'] == 'float' && !is_float($data)) {
                            Dever::error($v['message']);
                        } elseif ($v['type'] == 'string' && !is_string($data)) {
                            Dever::error($v['message']);
                        } elseif ($v['type'] == 'boolean' && !is_bool($data)) {
                            Dever::error($v['message']);
                        } elseif ($v['type'] == 'url' && !filter_var($data, FILTER_VALIDATE_URL)) {
                            Dever::error($v['message']);
                        } elseif ($v['type'] == 'email' && !filter_var($data, FILTER_VALIDATE_EMAIL)) {
                            Dever::error($v['message']);
                        } elseif ($v['type'] == 'enum' && isset($v['enum']) && !in_array($data, $v['enum'])) {
                            Dever::error($v['message']);
                        }
                    }
                    if (isset($v['len']) && $v['len'] && strlen($data) > $v['len']) {
                        Dever::error($v['message']);
                    }
                    if (isset($v['min']) && $v['min'] && strlen($data) < $v['min']) {
                        Dever::error($v['message']);
                    }
                    if (isset($v['max']) && $v['max'] && strlen($data) > $v['max']) {
                        Dever::error($v['message']);
                    }
                }
            }
        }
    }

    private function setData($setting, &$data, $field, $type, $disable)
    {
        $result = array();
        foreach ($setting as $k => $v) {
            if (!is_array($v)) {
                if (is_numeric($k)) {
                    $k = $v;
                    $v = $type;
                }
                if ($k == 'id') {
                    $v = array('name' => 'ID', 'type' => $v);
                } elseif ($k == 'cdate') {
                    $v = array('name' => '创建时间', 'type' => $v);
                } elseif(isset($this->db->config['struct'][$k])) {
                    $v = array('name' => $this->db->config['struct'][$k]['name'], 'type' => $v);
                } else {
                    $v = array('name' => $v);
                }
            } else {
                if (isset($v['only'])) {
                    if ($v['only'] == 'edit' && !$this->id) {
                        continue;
                    } elseif ($v['only'] == 'add' && $this->id) {
                        continue;
                    }
                }
            }
            if ($field) {
                if (is_string($field) && (strstr($field, '{') || strstr($field, '%7B'))) {
                    $field = htmlspecialchars_decode($field);
                    $field = Dever::json_decode($field);
                }
                if (is_array($field)) {
                    if (isset($field['param'])) {
                        if (isset($field['param']['set'])) {
                            $field = array_merge($field, $field['param']['set']);
                        }
                        if (isset($field['param']['search'])) {
                            $field = array_merge($field, $field['param']['search']);
                        }
                    } elseif (isset($field['field']) && !Dever::check($field['field'], $k)) {
                        continue;
                    }
                    if (isset($field[$k]) && $field[$k] != $k) {
                        $v['default'] = $field[$k];
                        $v['type'] = 'hidden';
                    }
                } elseif (!Dever::check($field, $k)) {
                    continue;
                }
            }
            $info = $this->setField($data, $k, $v, $field, $type, $disable);
            if ($info) {
                $result[] = $info;
            }
        }
        return $result;
    }

    private function setField(&$data, $key, $value, $field, $type = 'show', $disable = false)
    {
        $value['key'] = $key;
        $this->setName($value);
        # 对每个字段进行权限设置
        if (isset($value['func']) && $value['func']) {
            $func = $this->getFunc('field_' . $value['key'], '字段-' . $value['name'], 200);
            if (!$func) {
                return false;
            }
        }
        if (strpos($key, '/') && $this->key == 'update') {
            $this->setType($value, 'table');
            $this->setShow($value);
            if (strpos($key, '#')) {
                $key = str_replace('#', '', $key);
            }
            $sku = $value['type'] == 'sku' ? true : false;
            $value['value'] = $this->getOther($key, $value, $this->info);
            if (isset($value['default'])) {
                $value['value'] += $value['default'];
            }
            if (isset($value['field'])) {
                list($app, $table) = explode('/', $key);
                $set = Dever::project($app);
                $manage = @include($set['path'] . 'manage/'.$table.'.php');
                if (isset($manage['update']['field'][$value['field']])) {
                    $value = array_merge($value, $manage['update']['field'][$value['field']]);
                }
                if ($value['value']) {
                    $data[$key . '_id'] = array
                    (
                        'key' => $key . '_id',
                        'type' => 'hidden',
                        'value' => $value['value'][0]['id'],
                    );
                    $value['value'] = $value['value'][0][$value['field']];
                }
                $this->setRules($value);
                $this->setForm($value);
                $data[] = $value;
                return $value['name'] ?? 'test';
            }
            $update = new \Manage\Api\Page\Update($key, false);
            $value['option'] = array();
            $value['content'] = $update->get($value['value'], $value['option'], $sku);
            $data[] = $value;
            return $value['name'];
        } else {
            $this->setType($value, $type);
            $this->setDisable($value, $disable);
            if ($this->key == 'update') {
                # 一般为更新页面需要的参数
                $this->setShow($value);
                $this->setRules($value);
            } elseif (isset($value['remote'])) {
                $value['remote'] = Dever::url($value['remote']);
            }
            if ($type == 'show') {
                if (!isset($value['tip'])) {
                    $value['tip'] = false;
                }
                $in = array('switch', 'select', 'input');
                if (in_array($value['type'], $in)) {
                    $value['func'] = $this->getFunc('list_edit_' . $value['key'], '列表更新-' . $value['name'], 104);
                    if (!$value['func']) {
                        $value['type'] = 'show';
                        if (isset($value['show'])) {
                            unset($value['show']);
                        }
                    }
                }
                if (isset($value['child'])) {
                    $child = array();
                    $this->setData($value['child'], $child, $field, $type, $disable);
                    $value['child'] = $child;
                } else {
                    $this->field[$key] = $value;
                }
            }
            $this->setForm($value);
            $data[] = $value;
            return $value['name'] ?? 'test';
        }
    }

    private function setShow(&$value)
    {
        if ($value['type'] == 'hidden') {
            $value['show'] = false;
        } elseif (!isset($value['show'])) {
            $value['show'] = true;
        }
    }

    private function setName(&$value)
    {
        if (empty($value['name']) && isset($this->db->config['struct'][$value['key']])) {
            $value['name'] = $this->db->config['struct'][$value['key']]['name'];
        }
        if (empty($value['placeholder']) && isset($value['name'])) {
            $value['placeholder'] = $value['name'];
        }
    }

    private function setType(&$value, $type)
    {
        if (empty($value['type'])) {
            $value['type'] = $type;
        }
        if (strpos($value['type'], '(')) {
            $value['type'] = $type;
        }
        if (empty($value['width'])) {
            $value['width'] = 'auto';
        }
        if (isset($value['upload']) && Dever::project('upload')) {
            $upload['cate_id'] = 1;
            $upload['group_key'] = $this->db->config['table'] . '-' . $value['key'];
            $upload['group_name'] = $this->db->config['name'];
            $upload['user_token'] = Dever::load('common', 'manage')->getToken();
            $upload['user_table'] = $this->user['table'];
            $upload['user_id'] = $this->user['id'];
            if (is_array($value['upload'])) {
                $upload += $value['upload'];
            } else {
                $upload += array('id' => $value['upload']);
            }
            if (empty($upload['id'])) {
                Dever::error('上传配置错误');
            }
            $value['config'] = Dever::load('save', 'upload')->get($upload['id']);
            $value['yun'] = false;
            if ($value['config']['method'] == 2) {
                $value['yun'] = true;
            }
            $value['url'] = Dever::url('upload/save.act', $upload);
            $upload['wh'] = $value['wh'] ?? '500*500';
            $value['set'] = Dever::url('image/manage.set', $upload);
            if (isset($value['multiple']) && $value['multiple']) {
                $value['limit'] = 10;
            } else {
                $value['limit'] = 1;
            }
        }
        if (isset($value['editorMenu'])) {
            if (isset($value['editorMenu']['uploadImage'])) {
                if (!is_array($value['editorMenu']['uploadImage'])) {
                    $value['editorMenu']['uploadImage'] = array('upload' => $value['editorMenu']['uploadImage']);
                }
                $value['editorMenu']['uploadImage']['server'] = Dever::url('upload/save.wangEditor', array('id' => $value['editorMenu']['uploadImage']['upload']));
                if (empty($value['editorMenu']['uploadImage']['fieldName'])) {
                    $value['editorMenu']['uploadImage']['fieldName'] = 'file';
                }
            }
             if (isset($value['editorMenu']['uploadVideo'])) {
                if (!is_array($value['editorMenu']['uploadVideo'])) {
                    $value['editorMenu']['uploadVideo'] = array('upload' => $value['editorMenu']['uploadVideo']);
                }
                $value['editorMenu']['uploadVideo']['server'] = Dever::url('upload/save.wangEditor', array('id' => $value['editorMenu']['uploadVideo']['upload']));
                if (empty($value['editorMenu']['uploadImage']['fieldName'])) {
                    $value['editorMenu']['uploadImage']['fieldName'] = 'file';
                }
            }
        }

        if (isset($value['date_type']) && $value['date_type'] == 'datetimerange' && empty($value['default_time'])) {
            $value['default_time'] = array(\Dever\Helper\Date::mktime(date('Y-m-d 00:00:00'))*1000, \Dever\Helper\Date::mktime(date('Y-m-d 23:59:59'))*1000);
        }
    }

    private function setDisable(&$value, $disable)
    {
        if (isset($value['disable'])) {
            $disable = $value['disable'];
        }
        $value['disable'] = $disable;
    }

    private function setForm(&$value)
    {
        if (empty($value['value'])) {
            $value['value'] = Dever::input('search')[$value['key']] ?? '';
        }
        if (is_array($value['value'])) {
            foreach ($value['value'] as &$v) {
                $v = (float) $v;
            }
        }
        if (!$value['value']) {
            if (isset($value['default']) && !strstr($value['default'], '{')) {
                $value['value'] = $value['default'];
            } elseif ($this->key == 'update' && isset($this->db->config['struct'][$value['key']]['default'])) {
                $value['value'] = $this->db->config['struct'][$value['key']]['default'];
            }
        }
        if (isset($value['option']) && $value['option']) {
            $this->db->config['option'][$value['key']] = $value['option'];
        }
        if ($option = $this->db->value($value['key'])) {
            if ($value['type'] == 'checkbox') {
                $value['value'] = $value['value'] ? explode(',', $value['value']) : array();
            }
            $value['option'] = $option;
            if ($value['type'] == 'text') {
                $value['type'] = 'select';
            }
            if (!is_array($value['value']) && $value['value']) {
                $value['value'] = (float) $value['value'];
            }
        }
    }

    private function setRules(&$value)
    {
        if (isset($value['rules']) && $value['rules']) {
            if (!is_array($value['rules'])) {
                $value['rules'] = array
                (
                    array
                    (
                        'required' => true,
                        'trigger' => 'blur',
                        'message' => $value['name'] . '不能为空',
                    ),
                );
            }
            foreach ($value['rules'] as $k => $v) {
                if (isset($v['only'])) {
                    if ($v['only'] == 'edit' && !$this->id) {
                        unset($value['rules'][$k]);
                        break;
                    } elseif ($v['only'] == 'add' && $this->id) {
                        unset($value['rules'][$k]);
                        break;
                    }
                }
            }
            if (!isset($value['rules'][0])) {
                $value['rules'] = array_values($value['rules']);
            }
        }
    }

    protected function input($key, $value = '')
    {
        return $this->input ? Dever::input($key) : $value;
    }

    public function button($key = 'button', $data = array())
    {
        $result = array();
        if (!isset($this->config[$key])) {
            $num = 0;
            if (isset($this->db->config['manage']['update']['field'])) {
                $num = count($this->db->config['manage']['update']['field']);
            } elseif (isset($this->db->config['struct']) && $this->db->config['struct']) {
                $num = count($this->db->config['struct']);
            }
            $fast = 'fast';
            if ($num > 8) {
                $fast = '';
            }
            if ($key == 'button') {
                $this->config[$key] = array('新增' => $fast . 'add');
            } else {
                $this->config[$key] = array('编辑' => $fast . 'edit', '删除' => 'recycle');
            }
        }
        $sort = 1;
        foreach ($this->config[$key] as $k => $v) {
            $d = '';
            $p = '';
            $i = '';
            if (is_array($v)) {
                if (isset($v[3]) && $data) {
                    $d = $v[3];
                    parse_str($d, $t);
                    $state = true;
                    foreach ($t as $k1 => $v1) {
                        if (!isset($data[$k1]) || (isset($data[$k1]) && $data[$k1] != $v1)) {
                            $state = false;
                        }
                    }
                    if (!$state) {
                        continue;
                    }
                }
                if (isset($v[2])) {
                    $i = $v[2];
                }
                if (isset($v[1])) {
                    $p = $v[1];
                }
                $v = $v[0];
            }
            if (strstr($v, 'add')) {
                $icon = 'Plus';
                $button = 'primary';
            } elseif (strstr($v, 'edit')) {
                $icon = 'Edit';
                $button = 'primary';
            } elseif (strstr($v, 'view')) {
                $icon = 'View';
                $button = '';
            } elseif ($v == 'delete') {
                if ($key == 'button') {
                    if (isset($this->config['layout'])) {
                        continue;
                    }
                    $this->config['selection'] = true;
                }
                $icon = 'Delete';
                $button = 'danger';
            } elseif ($v == 'recycle') {
                if ($key == 'button') {
                    if (isset($this->config['layout'])) {
                        continue;
                    }
                    $this->config['selection'] = true;
                }
                $icon = 'Delete';
                $button = 'danger';
            } elseif ($v == 'oper') {
                if ($key == 'button') {
                    if (isset($this->config['layout'])) {
                        continue;
                    }
                    $this->config['selection'] = true;
                }
                $icon = 'Notification';
                $button = 'warning';
            } elseif ($v == 'api') {
                if ($key == 'button') {
                    if (isset($this->config['layout'])) {
                        continue;
                    }
                    $this->config['selection'] = true;
                }
                $p = Dever::url($p);
                $icon = 'Notification';
                $button = 'warning';
            } elseif ($v == 'link') {
                $icon = 'Link';
                $button = 'success';
            } elseif ($v == 'route') {
                $icon = 'Link';
                $button = 'success';
            } elseif ($v == 'recover') {
                $icon = 'CirclePlus';
                $button = 'info';
            } else {
                continue;
            }
            # 权限验证
            $sort++;
            $func = $this->getFunc($v, $k, $sort, $p);
            if ($this->menu && $this->menu['show'] == 1 && !$func) {
                continue;
            }
            if ($i) {
                $icon = $i;
            }
            $result[] = array
            (
                'name' => $k,
                'type' => $v,
                'param' => $p,
                'icon' => $icon,
                'button' => $button,
                'func' => $func,
            );
            if (!$this->recycler && $v == 'recycle') {
                $this->recycler = true;
            }
        }
        return $result;
    }
}